/*
 * Segment Datenübernahme und Aufbereitung (DUA), Fehleranalyse TLS spontan
 * Copyright (C) 2016 by Regierungspräsidium Tübingen, Landesstelle für Straßentechnik
 * Copyright 2016 by Kappich Systemberatung Aachen
 * Copyright (C) 2007 BitCtrl Systems GmbH
 * Basiert auf der gleichnamigen Klasse aus de.bsvrz.dua.fehlertls.tls, BitCtrl Systems GmbH
 *
 * SWE de.bsvrz.dua.fehlertlsspontan (Fehleranalyse TLS spontan)
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation; either version 3 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License along with this program;
 * if not, see <http://www.gnu.org/licenses/>.
 *
 * Dieses Programm ist freie Software. Sie können es unter den Bedingungen der GNU General Public
 * License, wie von der Free Software Foundation veröffentlicht, weitergeben und/oder modifizieren,
 * entweder gemäß Version 3 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
 * Die Veröffentlichung dieses Programms erfolgt in der Hoffnung, dass es Ihnen von Nutzen sein wird,
 * aber OHNE IRGENDEINE GARANTIE, sogar ohne die implizite Garantie der MARKTREIFE oder der
 * VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License.
 * Sie sollten ein Exemplar der GNU General Public License zusammen mit diesem Programm erhalten haben.
 * Falls nicht, siehe <http://www.gnu.org/licenses/>.
 * Eine deutsche Übersetzung findet sich unter http://www.gnu.de/documents/gpl.de.html.
 *
 * Contact Information/Kontakt:
 * Regierungspräsidium Tübingen
 * Landesstelle für Straßentechnik
 * Heilbronner Straße 300 - 302
 * 70469 Stuttgart
 * Phone: +49 0711 8910-200
 * mailto: abteilung9@rpt.bwl.de
 */

package de.bsvrz.dua.fehlertlsspontan.enums;

import java.util.HashMap;
import java.util.Map;

import de.bsvrz.sys.funclib.bitctrl.daf.AbstractDavZustand;

/**
 * Korrespondiert mit dem DAV-Enumerationstyp <code>att.tlsDEFehlerStatus</code>.
 *
 * @author BitCtrl Systems GmbH, Thierfelder
 */
public final class TlsDeFehlerStatus extends AbstractDavZustand {

	/**
	 * serialVersionUID.
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Der Wertebereich dieses DAV-Enumerationstypen.
	 */
	private static Map<Integer, TlsDeFehlerStatus> werteBereich = new HashMap<>();

	/**
	 * DE in Ordnung.
	 */
	public static final TlsDeFehlerStatus OK = new TlsDeFehlerStatus(
			"ok", 0, "DE in Ordnung");

	/**
	 * Störung vom E/A-Konzentrator erkannt.
	 */
	public static final TlsDeFehlerStatus STOER_EAK = new TlsDeFehlerStatus(
			"StörEAK", 1, "Störung vom E/A-Konzentrator erkannt");

	/**
	 * Störung vom SM erkannt.
	 */
	public static final TlsDeFehlerStatus STOER_SM = new TlsDeFehlerStatus(
			"StörSM", 2, "Störung vom SM erkannt");

	/**
	 * der Text der die Natur des DE-Fehlers illustriert.
	 */
	private String text = null;

	/**
	 * Standardkonstruktor.
	 *
	 * @param kode
	 *            der Kode
	 * @param name
	 *            die Bezeichnung
	 * @param text
	 *            der Text der die Natur des DE-Fehlers illustriert
	 */
	private TlsDeFehlerStatus(String name, int kode, String text) {
		super(kode, name);
		this.text = text;
		TlsDeFehlerStatus.werteBereich.put(kode, this);
	}

	/**
	 * Erfragt den Text der die Natur des DE-Fehlers illustriert.
	 *
	 * @return der Text der die Natur des DE-Fehlers illustriert
	 */
	public String getText() {
		return this.text;
	}

	/**
	 * Erfragt den Wert dieses DAV-Enumerationstypen mit dem übergebenen Code.
	 *
	 * @param kode
	 *            der Kode des Zustands
	 * @return der Enumerations-Wert
	 */
	public static TlsDeFehlerStatus getZustand(final int kode) {
		return TlsDeFehlerStatus.werteBereich.get(kode);
	}

}
