/*
 * Segment Datenübernahme und Aufbereitung (DUA), Fehleranalyse TLS spontan
 * Copyright (C) 2016 by Regierungspräsidium Tübingen, Landesstelle für Straßentechnik
 * Copyright 2016 by Kappich Systemberatung Aachen
 * Copyright (C) 2007 BitCtrl Systems GmbH
 * Basiert auf der gleichnamigen Klasse aus de.bsvrz.dua.fehlertls.tls, BitCtrl Systems GmbH
 *
 * SWE de.bsvrz.dua.fehlertlsspontan (Fehleranalyse TLS spontan)
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation; either version 3 of
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License along with this program;
 * if not, see <http://www.gnu.org/licenses/>.
 *
 * Dieses Programm ist freie Software. Sie können es unter den Bedingungen der GNU General Public
 * License, wie von der Free Software Foundation veröffentlicht, weitergeben und/oder modifizieren,
 * entweder gemäß Version 3 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
 * Die Veröffentlichung dieses Programms erfolgt in der Hoffnung, dass es Ihnen von Nutzen sein wird,
 * aber OHNE IRGENDEINE GARANTIE, sogar ohne die implizite Garantie der MARKTREIFE oder der
 * VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License.
 * Sie sollten ein Exemplar der GNU General Public License zusammen mit diesem Programm erhalten haben.
 * Falls nicht, siehe <http://www.gnu.org/licenses/>.
 * Eine deutsche Übersetzung findet sich unter http://www.gnu.de/documents/gpl.de.html.
 *
 * Contact Information/Kontakt:
 * Regierungspräsidium Tübingen
 * Landesstelle für Straßentechnik
 * Heilbronner Straße 300 - 302
 * 70469 Stuttgart
 * Phone: +49 0711 8910-200
 * mailto: abteilung9@rpt.bwl.de
 */

package de.bsvrz.dua.fehlertlsspontan.enums;

import java.util.HashMap;
import java.util.Map;

import de.bsvrz.sys.funclib.bitctrl.daf.AbstractDavZustand;

/**
 * Korrespondiert mit dem DAV-Enumerationstyp <code>att.tlsFehlerAnalyse</code>.
 *
 * @author Regierungspräsidium Tübingen, Landesstelle für Straßentechnik, Thomas Bräuner
 * @author BitCtrl Systems GmbH, Thierfelder
 */
public final class TlsFehlerAnalyse extends AbstractDavZustand {

	/**
	 * serialVersionUID.
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Der Wertebereich dieses DAV-Enumerationstypen.
	 */
	private static Map<Integer, TlsFehlerAnalyse> werteBereich = new HashMap<>();

	/**
	 * Fehlerursache konnte nicht ermittelt werden.
	 */
	public static final TlsFehlerAnalyse UNBEKANNT = new TlsFehlerAnalyse(
			"Unbekannte Ursache", 0);

	/**
	 * Verbindung zum KRI xxx oder KRI selbst defekt. Verbindung zum KRI oder
	 * KRI instand setzen
	 */
	public static final TlsFehlerAnalyse KRI_ALLE_DE_DEFEKT = new TlsFehlerAnalyse(
			"Alle DE am KRI defekt", 1);

	/**
	 * Modem am Inselbus xxx oder Inselbus selbst defekt. Modem oder Inselbus
	 * instand setzen
	 */
	public static final TlsFehlerAnalyse INSELBUS_ALLE_DE_DEFEKT = new TlsFehlerAnalyse(
			"Alle DE am Inselbus defekt", 2);

	/**
	 * Inselbus xxx gestört: Für die DE der Steuermodule x1, x2,.. sind keine
	 * Daten verfügbar. Inselbus xxx instand setzen
	 */
	public static final TlsFehlerAnalyse INSELBUS_DEFEKT = new TlsFehlerAnalyse(
			"Inselbus defekt", 3);

	/**
	 * Modem am Steuermodul x oder Steuermodul defekt. Modem am Steuermodul x
	 * oder Steuermodul instand setzen
	 */
	public static final TlsFehlerAnalyse SM_ALLE_DE_DEFEKT = new TlsFehlerAnalyse(
			"Alle DE am Steuermodul defekt", 4);

	/**
	 * EAK x am Steuermodul y defekt. EAK x am Steuermodul y instand setzen
	 */
	public static final TlsFehlerAnalyse EAK_ALLE_DE_DEFEKT = new TlsFehlerAnalyse(
			"Alle DE am EAK defekt", 5);

	/**
	 * Standardkonstruktor.
	 *
	 * @param kode
	 *            der Kode
	 * @param name
	 *            die Bezeichnung
	 */
	private TlsFehlerAnalyse(String name, int kode) {
		super(kode, name);
		TlsFehlerAnalyse.werteBereich.put(kode, this);
	}

	/**
	 * Erfragt den Wert dieses DAV-Enumerationstypen mit dem übergebenen Code.
	 *
	 * @param kode
	 *            der Kode des Zustands
	 * @return der Enumerations-Wert
	 */
	public static TlsFehlerAnalyse getZustand(final int kode) {
		return TlsFehlerAnalyse.werteBereich.get(kode);
	}

}
