/*
 * Segment Datenübernahme und Aufbereitung (DUA), Fehleranalyse TLS spontan
 * Copyright (C) 2016 by Regierungspräsidium Tübingen, Landesstelle für Straßentechnik
 * Copyright 2016 by Kappich Systemberatung Aachen
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * Basiert auf der gleichnamigen Klasse aus de.bsvrz.dua.fehlertls.tls, BitCtrl Systems GmbH
 * 
 * SWE de.bsvrz.dua.fehlertlsspontan (Fehleranalyse TLS spontan)
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of the 
 * GNU General Public License as published by the Free Software Foundation; either version 3 of 
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 *
 * Dieses Programm ist freie Software. Sie können es unter den Bedingungen der GNU General Public 
 * License, wie von der Free Software Foundation veröffentlicht, weitergeben und/oder modifizieren, 
 * entweder gemäß Version 3 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
 * Die Veröffentlichung dieses Programms erfolgt in der Hoffnung, dass es Ihnen von Nutzen sein wird, 
 * aber OHNE IRGENDEINE GARANTIE, sogar ohne die implizite Garantie der MARKTREIFE oder der 
 * VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License.
 * Sie sollten ein Exemplar der GNU General Public License zusammen mit diesem Programm erhalten haben. 
 * Falls nicht, siehe <http://www.gnu.org/licenses/>.
 * Eine deutsche Übersetzung findet sich unter http://www.gnu.de/documents/gpl.de.html.
 *
 * Contact Information/Kontakt:
 * Regierungspräsidium Tübingen
 * Landesstelle für Straßentechnik
 * Heilbronner Straße 300 - 302
 * 70469 Stuttgart
 * Phone: +49 0711 8910-200
 * mailto: abteilung9@rpt.bwl.de
 */

package de.bsvrz.dua.fehlertlsspontan.tls;

import java.util.HashSet;
import java.util.Set;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dua.fehlertlsspontan.enums.TlsFehlerAnalyse;
import de.bsvrz.sys.funclib.operatingMessage.MessageGrade;
import de.bsvrz.sys.funclib.operatingMessage.MessageTemplate;
import de.bsvrz.sys.funclib.operatingMessage.MessageType;
import de.bsvrz.sys.funclib.operatingMessage.OperatingMessage;

/**
 * TLS-Hierarchieelement KRI.
 * 
 * @author BitCtrl Systems GmbH, Thierfelder
 */
public class Kri extends TlsGeraet {

	private static final MessageTemplate TEMPLATE = new MessageTemplate(
			MessageGrade.ERROR,
			MessageType.APPLICATION_DOMAIN,
	        MessageTemplate.fixed("Alle DE oder EAK am KRI "),
	        MessageTemplate.object(),
	        MessageTemplate.fixed(" defekt. DE und EAK am KRI instand setzen. "),
	        MessageTemplate.ids()
	).withIdFactory(message -> message.getObject().getPidOrId() + " [DUA-FS-FU]");
	
	/**
	 * Standardkonstruktor.
	 * 
	 * @param dav
	 *            Datenverteiler-Verbindung
	 * @param objekt
	 *            ein Systemobjekt vom Typ <code>typ.kri</code>
	 * @param elter
	 *            das in der TLS-Hierarchie über diesem Gerät liegende Gerät
	 */
	protected Kri(ClientDavInterface dav, SystemObject objekt, TlsGeraet elter) {
		super(dav, objekt, elter);

		/**
		 * Initialisiere Inselbusse
		 */
		// Das sind keine Inselbusse!
//		for (SystemObject inselBus : this.objekt.getNonMutableSet("AnschlussPunkteGerät").getElements()) { 
//			if (inselBus.isValid()) {
//				this.kinder.add(new Inselbus(dav, inselBus, this));
//			}
//		}
		
		// Inselbusse sind die Ports der AnschlussPunkteGerät
		final AttributeGroup atgAnschlussPunkt = dav.getDataModel().getAttributeGroup("atg.anschlussPunkt");
		final Set<String> ports = new HashSet<>();
		for (final SystemObject apg : this.objekt.getNonMutableSet("AnschlussPunkteGerät").getElements()) {
			if (apg.isValid()) {
				ports.add(apg.getConfigurationData(atgAnschlussPunkt).getTextValue("PortNummer").getText());
			}
		}
		for (String p : ports) {
			for (final SystemObject apg : this.objekt.getNonMutableSet("AnschlussPunkteGerät").getElements()) {
				if (apg.isValid()) {
					if (apg.getConfigurationData(atgAnschlussPunkt).getTextValue("PortNummer").getText().equals(p)) {
						this.kinder.add(new Inselbus(dav, apg, p, this));
						break;
					}
				}
			}
		}
	}

	/**
	 * {@inheritDoc}
	 * <br>In diesem Fall ein KRI.
	 */
	@Override
	public Art getGeraeteArt() {
		return Art.KRI;
	}

	/**
	 * {@inheritDoc}
	 * In diesem Fall alle DE eines KRI defekt.
	 */
	@Override
	public void publiziereFehler(long zeitStempel) {
		OperatingMessage message = TEMPLATE.newMessage(objekt);
		message.addId("[DUA-FS-FU01]");
		if (darfBetriebsmeldungSenden())
			this.publiziere(message);

		for (De de : getDes()) {
			if (de.getObjekt().isOfType("typ.deWzg") || de.getObjekt().isOfType("typ.deVlt")) {
				de.publiziereFehlerUrsache(zeitStempel,	TlsFehlerAnalyse.KRI_ALLE_DE_DEFEKT);
			}
		}
	}
	
	/**
	 * Bestimmt, ob ein Inselbus eine Betriebsmeldung absetzen darf
	 * oder ob das der KRI macht, weil alle DE auf Störung sind.
	 * 
	 * @param zeitStempel
	 *            der Zeitstempel des Fehlers
	 *            
	 * @return true, wenn alle DE gestört sind, sonst false
	 */
	public boolean alleDeGestoert(long zeitStempel) {
		boolean erg = true;
		for (De de : getDes()) {
			if (!de.isGestoert() && de.isInitialisiert()) {
				erg = false;
			}
		}
		if (erg) {
			// Es sind alle DE am KRI gestört, so dass der KRI den Fehler publiziert
			publiziereFehler(zeitStempel);
		}
		return erg;
	}

}
