/*
 * Segment Datenübernahme und Aufbereitung (DUA), Fehleranalyse TLS spontan
 * Copyright (C) 2016 by Regierungspräsidium Tübingen, Landesstelle für Straßentechnik
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * Basiert auf der gleichnamigen Klasse aus de.bsvrz.dua.fehlertls.tls, BitCtrl Systems GmbH
 * 
 * SWE de.bsvrz.dua.fehlertlsspontan (Fehleranalyse TLS spontan)
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of the 
 * GNU General Public License as published by the Free Software Foundation; either version 3 of 
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 *
 * Dieses Programm ist freie Software. Sie können es unter den Bedingungen der GNU General Public 
 * License, wie von der Free Software Foundation veröffentlicht, weitergeben und/oder modifizieren, 
 * entweder gemäß Version 3 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
 * Die Veröffentlichung dieses Programms erfolgt in der Hoffnung, dass es Ihnen von Nutzen sein wird, 
 * aber OHNE IRGENDEINE GARANTIE, sogar ohne die implizite Garantie der MARKTREIFE oder der 
 * VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License.
 * Sie sollten ein Exemplar der GNU General Public License zusammen mit diesem Programm erhalten haben. 
 * Falls nicht, siehe <http://www.gnu.org/licenses/>.
 * Eine deutsche Übersetzung findet sich unter http://www.gnu.de/documents/gpl.de.html.
 *
 * Contact Information/Kontakt:
 * Regierungspräsidium Tübingen
 * Landesstelle für Straßentechnik
 * Heilbronner Straße 300 - 302
 * 70469 Stuttgart
 * Phone: +49 0711 8910-200
 * mailto: abteilung9@rpt.bwl.de
 */

package de.bsvrz.dua.fehlertlsspontan.tls;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

import java.util.HashSet;
import java.util.Set;

/**
 * Initialisiert alle Objekte im Teilmodel TLS, die (inklusive und) unterhalb
 * der uebergebenen Objekte vom Typ <code>typ.gerät</code> konfiguriert sind.
 * 
 * @author BitCtrl Systems GmbH, Thierfelder
 */
public final class TlsHierarchie extends TlsGeraet {

	/**
	 * Datenverteiler-Verbindung.
	 */
	private ClientDavInterface dav = null;

	/**
	 * Konfigurierende Eigenschaften eines Kommunikationspartners an einem
	 * Anschlusspunkt.
	 */
	public AttributeGroup konfigAtg = null;

	/**
	 * Standardkonstruktor.
	 * 
	 * @param dav
	 *            Datenverteiler-Verbindung
	 */
	private TlsHierarchie(ClientDavInterface dav) {
		super(dav, null, null);
		this.dav = dav;
		this.konfigAtg = dav.getDataModel().getAttributeGroup("atg.anschlussPunktKommunikationsPartner");
	}

	/**
	 * Standardkonstruktor.
	 *  @param dav
	 *            Datenverteiler-Verbindung
	 * @param geraete
	 * 			  Geräte, die in der Kommandozeile uebergeben wurden
	 * 
	 * @return die Instanz der TLS-Hierarchie
	 */
	public static TlsHierarchie getInstance(ClientDavInterface dav, Set<SystemObject> geraete) {
		TlsHierarchie wurzel;
		wurzel = new TlsHierarchie(dav);

		for (SystemObject geraet : geraete) {
			wurzel.initialisiere((ConfigurationObject) geraet);
		}
		return wurzel;
	}

	/**
	 * Initialisiert ein einzelnes Objekt vom Typ <code>typ.gerät</code>.
	 * 
	 * @param geraet
	 *            ein Objekt vom Typ <code>typ.gerät</code>
	 */
	private void initialisiere(ConfigurationObject geraet) {
		if (geraet.isOfType("typ.kri2_b")) {
			// Vor SM prüfen, da Kri2b auch SM sind
			kinder.add(new Kri(dav, geraet, this));
		} else if (geraet.isOfType("typ.steuerModul")) {
			kinder.add(new Sm(dav, geraet, this));
		} else if (geraet.isOfType("typ.kri")) { 
			kinder.add(new Kri(dav, geraet, this));
		} else if (geraet.isOfType("typ.uz") ||
				geraet.isOfType("typ.viz") ||
				geraet.isOfType("typ.vrz")) {
			for (SystemObject anschlussPunktSysObj : geraet.getNonMutableSet(
					"AnschlussPunkteGerät").getElements()) { 
				if (anschlussPunktSysObj.isValid()) {
					ConfigurationObject anschlussPunktKonObj = (ConfigurationObject) anschlussPunktSysObj;

					Set<SystemObject> unterGeraete = new HashSet<>();
					for (SystemObject komPartner : anschlussPunktKonObj
							.getNonMutableSet("AnschlussPunkteKommunikationsPartner").getElements()) {

						Data konfigDatum = komPartner.getConfigurationData(konfigAtg);
						if (konfigDatum != null) {
							SystemObject unterGeraet = konfigDatum.getReferenceValue("KommunikationsPartner")
									.getSystemObject();
							if (unterGeraet != null) {
								unterGeraete.add(unterGeraet);
							} else {
								Debug.getLogger().warning("An " + komPartner + " (Gerät: " + geraet +
										") ist kein Gerät definiert");
							}
						} else {
							Debug.getLogger().warning("Konfiguration von " + komPartner + " (an Geraet: " + 
									geraet + ") konnte nicht ausgelesen werden. Das assoziierte Geraet wird ignoriert.");
						}
					}

					/**
					 * Iteriere über alle Untergeräte dieses Anschlusspunktes.
					 * Wenn ALLE Anschlusspunkte Steuermodule sein sollten, dann
					 * wird davon ausgegangen, dass es sich bei diesem
					 * Anschlusspunkt um einen Inselbus handelt.
					 */
					int steuerModulZaehler = 0;
					for (SystemObject unterGeraet : unterGeraete) {
						if (unterGeraet.isOfType("typ.steuerModul") && !unterGeraet.isOfType("typ.kri2_b")) {
							steuerModulZaehler++;
						}
					}

					if (unterGeraete.size() > 0) {
						if (unterGeraete.size() == steuerModulZaehler) {
							kinder.add(new Inselbus(dav, anschlussPunktSysObj, this));
						} else {
							for (SystemObject unterGeraet : unterGeraete) {
								initialisiere((ConfigurationObject) unterGeraet);
							}
						}
					}
				}
			}
		}
	}

	/**
	 * {@inheritDoc}
	 * <br>In diesem Fall nichts.
	 */
	@Override
	public Art getGeraeteArt() {
		return null;
	}

	/**
	 * {@inheritDoc}
	 * <br>In diesem Fall keinen.
	 */
	@Override
	public void publiziereFehler(long zeitStempel) {
		assert (false);
	}

}
